!     ------------------------------------------------------------------
!
!     main.f95
!
!
!     Created on Sat Mar 11 04:07:51 2006
!     Copyright (c) 2006 James K Beard. All rights reserved.
!
!
!     ------------------------------------------------------------------
program main	!Test double precision log(gamma(x)), incomplete beta function
use probability_distributions
use pauses
implicit none
integer,parameter::lunout=10
integer::iopt,ierr
real(kind(1.d0))::x,y,z,p,a,b,nu,nu1,nu2,lambda,n

  print 1000
  if(.NOT.yes_or_no("OK?")) stop

  print *,"Output in file logtest.txt"

  open(lunout,file="logtest.txt")

  do
    do
      print 2000
      read(*,fmt=3000,iostat=ierr) iopt
      if(ierr /= 0) then
        print *,"Input must be valid integer"
      else
        exit
      end if
    end do
    select case(iopt)
!----------------------------------------------
    case (0)
      exit
!----------------------------------------------
    case (1)
      call gentest
      exit

!----------------------------------------------
    case (2)
      call writelog("Log Gamma function")
      x=getx(.TRUE.)
      y=log_gamma(x)
      z=exp(y)
      call writelog("x, log(Gamma(x)), Gamma(x)")
      print *,x,y,z; write(lunout,*) x,y,z

!----------------------------------------------
    case (3)
      call writelog("Inverse Gaussian distribution function")
      x=getp()
      y=qgaussinv(x); z=qgaussinv(1.d0-x); p=qgauss(y)
      call writelog("p, Inverse P(p), inverse Q(p), Q(inverse Q(p)):")
      print *,x,z,y,p; write(lunout,*) x,z,y,p

      call writelog("Gaussian distribution function")
      x=getx(.FALSE.)
      y=qgauss(x); z=qgauss(-x); p=qgaussinv(y)
      call writelog("x, P(x), Q(x), inverse Q(Q(x)):")
      print *,x,z,y,p; write(lunout,*) x,z,y,p

!----------------------------------------------
    case (4)
      call writelog("Inverse chi-square distribution function")
      x=getp()
      nu=getDOF()

      y=Inv_qchisq_pdf(x,nu); z=Inv_qchisq_pdf(1.d0-x,nu); p=chisqpdf(y,nu,.TRUE.)

      call writelog("Number of degrees of freedom:")
      print *,nu; write(lunout,*) nu
      call writelog("p,Inverse P(p), inverse Q(p), Q(inverse Q(p)):")
      print *,x,z,y,p; write(lunout,*) x,z,y,p

      call writelog("Chi-square distribution function")
      x=getx(.TRUE.)
      nu=getDOF()

      y=chisqpdf(x,nu,.FALSE.); z=chisqpdf(x,nu,.TRUE.); p=Inv_qchisq_pdf(z,nu)

      call writelog("Number of degrees of freedom:")
      print *,nu; write(lunout,*) nu
      call writelog("x,P(x,nu), Q(x,nu), Q(inverse Q(x,nu)):")
      print *,x,y,z,p; write(lunout,*) x,y,z,p

!----------------------------------------------
    case(5)
      call writelog("Non-central chi-square distribution function")
      x=getx(.TRUE.)
      nu=getDOF()
      do
        print *,"Please input the non-centrality parameter (floating point)"
        read(*,fmt=*,iostat=ierr) lambda
        if(ierr /= 0) then
          print *,"Please input valid positive floating point number"
        elseif(nu <= 0.d0) then
          print *,"Non-centrality paramter must be greater than 0"
          print *,"Please try again"
        else
          exit
        end if
      end do

      y=ncchisq(x,nu,lambda,.FALSE.); z=ncchisq(x,nu,lambda,.TRUE.)

      call writelog("Number of degrees of freedom:")
      print *,nu; write(lunout,*) nu
      call writelog("Non-centrality parameter:")
      print *,lambda; write(lunout,*) lambda
      call writelog("x,P(x,nu,lambda), Q(x,nu,lambda):")
      print *,x,y,z; write(lunout,*) x,y,z
!----------------------------------------------
    case (6)
      call writelog("Inverse Student's t distribution")
      x=getp()
      nu=getDOF()

      y=InvStudents_tQ(x,nu); z=InvStudents_tQ(1.d0-x,nu); p=Students_tQ(y,nu)

      call writelog("Number of degrees of freedom nu:")
      print *,nu; write(lunout,*) nu1
      call writelog("p,Inverse A(p), inverse AQ(p), AQ(inverse AQ(p)):")
      print *,x,z,y,p
      write(lunout,*) x,z,y,p

      call writelog("Student's t distribution")
      x=getx(.TRUE.)
      nu=getDOF()

      y=Students_tP(x,nu); z=Students_tQ(x,nu); p=InvStudents_tQ(z,nu)

      call writelog("Number of degrees of freedom nu:")
      print *,nu; write(lunout,*) nu1
      call writelog("x,P(x,nu), Q(x,nu), Q(inverse Q(x,nu)):")
      print *,x,y,z,p
      write(lunout,*) x,y,z,p
!----------------------------------------------
    case (7)
      call writelog("Inverse F-distribution")
      x=getp()
      do
        print *,"Please input the two DOFs (floating point) separated by commas:"
        read(*,fmt=*,iostat=ierr) nu1,nu2
        if(ierr /= 0) then
          print *,"Please input valid positive floating point numbers."
        elseif(nu1 <= 0.d0 .OR. nu2 <= 0.d0) then
          print *,"Degrees of freedom must both be greater than 0."
          print *,"Please try again."
        else
          exit
        end if
      end do

      y=InvF_distQ(x,nu1,nu2); z=InvF_distQ(1.d0-x,nu1,nu2); p=F_distQ(y,nu1,nu2)

      call writelog("Numbers of degrees of freedom nu1, nu2:")
      print *,nu1,nu2; write(lunout,*) nu1,nu2
      call writelog("p,Inverse F(p,nu1,nu2), inverse FQ(p,nu1,nu2), FQ(inverse FQ(p,nu1,nu2)):")
      print *,x,z,y,p
      write(lunout,*) x,y,z,p

      call writelog("F-distribution")
      x=getx(.TRUE.)
      do
        print *,"Please input the two DOFs (floating point) separated by commas:"
        read(*,fmt=*,iostat=ierr) nu1,nu2
        if(ierr /= 0) then
          print *,"Please input valid positive floating point number"
        elseif(nu1 <= 0.d0 .OR. nu2 <= 0.d0) then
          print *,"Degreess of freedom must both be greater than 0"
          print *,"Please try again"
        else
          exit
        end if
      end do

      y=F_distP(x,nu1,nu2); z=F_distQ(x,nu1,nu2); p=InvF_distQ(z,nu1,nu2)

      call writelog("Numbers of degrees of freedom nu1, nu2:")
      print *,nu1,nu2; write(lunout,*) nu1,nu2
      call writelog("x,P(x,nu), Q(x,nu), Q(inverse Q(x,nu)):")
      print *,x,y,z,p
      write(lunout,*) x,y,z,p
!----------------------------------------------
    case (8)
      call writelog("Binomial distribution sum(i=0 to a-1): (n!/(i!*(n-i)!)*p^i*(1-p)^n-i)")
      do
        print *,"Please input probability parameter p"
        read(*,fmt=*,iostat=ierr) x
        if(ierr /= 0) then			!Bad input?
          print *,"Please input valid nonnegative floating point number"
        elseif(x < 0.0d0) then	!Valid range?
          print *,"Number for P(x,nu) and Q(x,nu) must be nonnegative"
          print *,"Please try again."
        else
          exit					!Good input
        end if
      end do
      do
        print *,"Please input the numbers a and n (floating point) separated by commas:"
        read(*,fmt=*,iostat=ierr) a,n
        if(ierr /= 0) then
          print *,"Please input valid positive floating point number"
        elseif(a == 0.d0 .OR. n < 0.d0) then
          print *,"Bith a and n must be greater than 0"
          print *,"Please try again"
        else
          exit
        end if
      end do

      y=binomial_distP(x,a,n); z=binomial_distQ(x,a,n)

      call writelog("Binomial probability parameter p:")
      print *,x; write(lunout,*) x
      call writelog("Sum parameters a and n:")
      print *,a,n; write(lunout,*) a,n
      call writelog("P(a,n), Q(a,n):")
      print *,y,z; write(lunout,*) y,z
!----------------------------------------------
    case (9)
      call writelog("Inverse incomplete beta function")
      x=getp()
      do
        print *,"Please input the degrees of freedom paramters a and b (floating point) separated by commas:"
        read(*,fmt=*,iostat=ierr) a,b
        if(ierr /= 0) then
          print *,"Please input valid positive floating point numbers"
        elseif(a <= 0.d0 .OR. b <= 0.d0) then
          print *,"Degrees of freedom parameters must both be greater than 0"
          print *,"Please try again"
        else
          exit
        end if
      end do

      y=invbetaf(x,a,b); z=invbetaf(1.d0-x,a,b); p=betaf(y,a,b)

      call writelog("Degrees of freedom parameters a, b:")
      print *,a,b; write(lunout,*) a,b
      call writelog("x,Inverse I(x,a,b), inverse IQ(x,a,b), I(inverse FQ(p,a,b),a,b):")
      print *,x,z,y,p
      write(lunout,*) x,z,y,p

      call writelog("Incomplete beta function")
      do
        print *,"Please input the paramter x between 0 and 1 (inclusive)"
        read(*,fmt=*,iostat=ierr) x
        if(ierr /= 0) then			!Bad input?
          print *,"Please input valid nonnegative floating point number"
        elseif(abs(x-.5d0) > 0.5d0) then	!Valid range?
          print *,"Number for paramter x must not be negative or >1"
          print *,"Please try again."
        else
          exit					!Good input
        end if
      end do
      do
        print *,"Please input the degrees of freedom parameters a and b (floating point) separated by commas:"
        read(*,fmt=*,iostat=ierr) a,b
        if(ierr /= 0) then
          print *,"Please input valid positive floating point numbers"
        elseif(a <= 0.d0 .OR. b <= 0.d0) then
          print *,"Degreess of freedom paramters must both be greater than 0"
          print *,"Please try again"
        else
          exit
        end if
      end do

      y=betaf(x,a,b); z=1.d0-betaf(x,a,b); p=invbetaf(y,a,b)

      call writelog("Degrees of freedom paramters a, b:")
      print *,a,b; write(lunout,*) a,b
      call writelog("x, I(x,a,b), 1-I(x,a,b), I(inverse I(x,a,b)a,b):")
      print *,x,y,z,p
      write(lunout,*) x,y,z,p
!----------------------------------------------
    case default
      print *,"Option not in range:",iopt
    end select

  end do

  close(lunout)
!-------------------------------------------------------------------------------
1000 format("Program, outputs (except user-determined numerical content)"/	&
 "copyright (c) 2006 by James K Beard.  All rights reserved.  You agree to"/	&
 "hold James K Beard and all other entities harmless in your use of this"/	&
 "software?")
2000 format("Options:"/	&
  "0  Exit program"/	&
  "1  Run general test and exit"/	&
  "2  Use logarithm of Gamma function"/	&
  "3  Use Gaussian distribution"/	&
  "4  Use Chi-square distribution"/	&
  "5  Use Non-central chi-square distributino"/	&
  "6  Use Student's t distribution"/	&
  "7  Use F-distribution"/	&
  "8  Use binomial distribution"/	&
  "9  Use Incomplete beta function"/	&
  "Your selection?"/)

3000 format(i)
!===============================================================================
contains
!===============================================================================
subroutine writelog(str)	!Writes string to run log, echoes to screen
character(len=*),intent(IN)::str
  print *,trim(str)
  write(lunout,'(a)') trim(str)
end subroutine writelog
!===============================================================================
real(kind(1.d0)) function getp()
  do
    print *,"Please input a probability or confidence level (greater than 0, less than 1):"
    read(*,fmt=*,iostat=ierr) getp
    if(ierr /= 0) then
      print *,"Please input a valid floating point number between 0 and 1:"
    elseif(abs(getp-0.5d0) > 0.5d0) then
      print *,"Probability must be greater than 0.0 and less than 1.00;"
    else
      exit
    end if
    print *,"Please try again."
  end do
end function getp
!===============================================================================
real(kind(1.d0)) function getx(positive)
logical,intent(IN)::positive
  do
    if(positive) then
      print *,"Please input a nonnegative floating point variable limit X:"
    else
      print *,"Please input a floating point variable limit X:"
    end if
    read(*,fmt=*,iostat=ierr) getx
    if(ierr /= 0) then
      if(positive) then
        print *,"Please input a valid nonnegative floating point number"
      else
        print *,"Please input a valid floating point number"
      end if
    elseif(((positive)) .AND. (getx < 0.d0)) then
      print *,"Variable limit X must be nonnegative."
    else
      exit
    end if
    print *,"Please try again."
  end do
end function getx
!===============================================================================
real(kind(1.d0)) function getDOF()
  do
    print *,"Please input the number of degrees of freedom (floating point):"
    read(*,fmt=*,iostat=ierr) getDOF
    if(ierr /= 0) then
      print *,"Please input valid positive floating point number."
    elseif(getDOF <= 0.d0) then
      print *,"Degrees of freedom must be greater than 0."
      print *,"Please try again."
    else
      exit
    end if
  end do
end function getDOF
!===============================================================================
subroutine gentest
implicit none
  real(kind(1d0))::x,dx=0.05d0,y,z,p,nu1,nu2,chisq
  integer::i

  write(lunout,*) "x, ln(Gamma(x)), Gamma(x):"
!Test log gamma function
  do i=1,20
    x=1d0+i*dx
    y=log_gamma(x)
    write(lunout,*) x,y,exp(y)
  end do

!Test F-distribution
  write(lunout,*)
  write(lunout,*) "F-distribution Q = 0.5; A&S p. 986"
  p=0.5d0
  nu1=1.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=60.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=3.d0
  nu2=30d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)

  write(lunout,*)
  write(lunout,*) "F-distribution Q = 0.25, A&S p. 986"
  p=0.25d0
  nu1=1.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=60.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=3.d0
  nu2=30d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)

  write(lunout,*)
  write(lunout,*) "F-distribution Q = 0.001, A&S p. 989"
  p=0.001d0
  nu1=1.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=60.d0
  nu2=1.d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)
  nu1=3.d0
  nu2=30d0
  write(lunout,*) "nu1, nu2:",nu1,nu2
  z=InvF_distQ(p,nu1,nu2)
  write(lunout,*) "Inverse:",z
  write(lunout,*) "P(z,nu1,nu2):",F_distP(z,nu1,nu2)
  write(lunout,*) "Q(z,nu1,nu2):",F_distQ(z,nu1,nu2)

  write(lunout,*)
  write(lunout,*) "Student's t distribution, A&S p. 990"
  p=0.2d0
  nu1=1d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=5d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=30d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)

  write(lunout,*)
  p=0.5d0
  nu1=1d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=5d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=30d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)

  write(lunout,*)
  p=0.999999d0
  nu1=1d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=5d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)
  nu1=30d0
  z=InvStudents_tQ(1.d0-p,nu1)
  write(lunout,*) "p, nu:",p,nu1
  write(lunout,*) "Inverse:",z
  write(lunout,*) "AP(z,nu):",Students_tP(z,nu1)
  write(lunout,*) "AQ(z,nu):",Students_tQ(z,nu1)

  write(lunout,*)
  write(lunout,*) "Chi-square distribution, A&S p. 978"
  chisq=2.d0
  nu1=1.d0
  write(lunout,*) "chisq,nu:",chisq,nu1
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  nu1=5.d0
  write(lunout,*) "chisq,nu:",chisq,nu1
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  nu1=10.d0
  write(lunout,*) "chisq,nu:",chisq,nu1
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  nu1=16.d0
  write(lunout,*) "chisq,nu:",chisq,nu1
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)

  write(lunout,*)
  p=.5d0
  nu1=1.d0
  chisq=Inv_qchisq_pdf(p,nu1)
  write(lunout,*) "p,nu,chisq:",p,nu1,chisq
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  p=.01d0
  nu1=10.d0
  chisq=Inv_qchisq_pdf(p,nu1)
  write(lunout,*) "p,nu,chisq:",p,nu1,chisq
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  p=.001d0
  nu1=10.d0
  chisq=Inv_qchisq_pdf(p,nu1)
  write(lunout,*) "p,nu,chisq:",p,nu1,chisq
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)
  p=.000001d0
  nu1=10.d0
  chisq=Inv_qchisq_pdf(p,nu1)
  write(lunout,*) "p,nu,chisq:",p,nu1,chisq
  write(lunout,*) "P(chisq,nu):",chisqpdf(chisq,nu1,.FALSE.)
  write(lunout,*) "Q(chisq,nu):",chisqpdf(chisq,nu1,.TRUE.)

end subroutine gentest
end program main
